/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 2    MBSETLIN.CPP    25-Feb-94    18:53:02 #    */

/* ************************************************************* *
 *  MBSETLIN.C  -  MetaBGI setlinestyle() function               *
 *  Copyright (c) 1994 Metagraphics Software Corporation         *
 * ************************************************************* */


#include "metabgi.h"


void far _Cdecl setlinestyle ( int      lstyle,  // line style (0-4)
                               unsigned upatt,   // dash pattern (lstyle=4)
                               int      lthick ) // line thickness
{
   static dashRcd usrDash;             // user-defined line style
   static Byte    usrCnts[16];

   int i, run, runlen, state;

   if( lstyle < 0 || lstyle > USERBIT_LINE ) {
      metabgi.bgiErrCode = grError;
      return;      
   }

   // if defining a solid dash, that's real stupid
   if( lstyle == USERBIT_LINE && upatt >= 0xFFFF )
      lstyle = 0;

   // save the BGI line styles
   metabgi.linesettings.linestyle = lstyle;
   metabgi.linesettings.upattern  = upatt;
   metabgi.linesettings.thickness = lthick;


   // set the dash style
   if( lstyle == USERBIT_LINE ) {

      // define dash style 7 to user pattern

      // convert from 16 bit mask to run lengths
      usrDash.dashSize = 0; 
      run = 1;       // start as initially in an on run      
      runlen = 0;    // of 0 length
      for( i = 0; i < 16; i++ ) {

         // isolate state of lsb
         if( upatt & 1 )
             state = 1;
         else
             state = 0;

         // when state transitions
         if( state != run ) {
            // add a new dash run sequence            
            usrCnts[usrDash.dashSize++] = runlen;
            // start a new run
            run = state;
            runlen = 1;
         }
          else
            runlen++;
   
         // next bit of mask
         upatt = upatt >> 1;            
      }

      // add terminating dash sequence
      usrCnts[usrDash.dashSize++] = runlen;

      // always force an even number of dash list entries
      if( usrDash.dashSize & 1 ) 
         usrCnts[usrDash.dashSize++] = 0;

      // define this dash style
      usrDash.dashList = usrCnts; 
      DefineDash( 7, &usrDash );
      metabgi.mwLineDash = 7;
   } 
   else {    // not a user defined dash
      metabgi.mwLineDash = lstyle;
      if ( lthick > 1 && lstyle > 0 )    // if wide and dashed
         metabgi.mwLineDash += 3;   // use wide versions of dash style
   }

   
   PenDash ( metabgi.mwLineDash );


   // set the line thickness
   if ( lthick <= 1 )    // check if single width
      {
      metabgi.mwLineThick = 0;
      PenSize ( 0, 0 );  // do optimized single width lines
      }
   else
      {
      metabgi.mwLineThick = lthick;
      PenSize ( lthick, lthick );
      }

   // clear the line style flags
   metabgi.mwFlags &= ~( flgLineStyle1 + flgLineStyle2 ) ;


} /* setlinestyle() */


/* End of File - MBSETLIN.C */

