/*  %kw # %v    %n    %d    %t #                      */
/*  Version # 3    MBSETTXS.CPP    10-Mar-94    18:04:42 #    */

/* ************************************************************* *
 *  MBSETTXS.C  -  MetaBGI settextstyle() function               *
 *  Copyright (c) 1994 Metagraphics Software Corporation         *
 * ************************************************************* */


#include "metabgi.h"

static char far * fontFile[maxStdFonts] = {
   {"BGIN__01.FNT"},    // default bitmapped font
   {"TRIP.FNT"},    // triplex font
   {"LITT.FNT"},    // small font
   {"SANS.FNT"},    // sans serif 
   {"GOTH.FNT"},    // gothic
   {"SCRI.FNT"},    // script
   {"SIMP.FNT"},    // simplex script
   {"TSCR.FNT"},    // triplex script    
   {"LCOM.FNT"},    // complex
   {"EURO.FNT"},    // european
   {"BOLD.FNT"}     // bold
};

// ratios for computing stroked font sizes 0 to 10
static point strksz[11] = { 
{0,0}, {3,5}, {2,3}, {3,4}, {1,1}, {4,3}, {5,3}, {2,1}, {5,2}, {3,1}, {4,1}
};

void far _Cdecl settextstyle ( int font, int direction, int charsize )
{
   dirRec   fileInfo;
   fontRcd *newfnt;
   int      i, registered;
   char     buf[15];
   void     **p;


   // make sure passed font number is in range
 	if( font < 0 || font >= (maxUserFont + maxStdFonts) ) {
      font = 0;
      metabgi.bgiErrCode = grInvalidFontNum;
   }

   // should we load a new font file ?
   if ( metabgi.textsettings.font != font || font == 0 ) {

      metabgi.textsettings.font = font;

      // toss the old font if it was ever allocated
      if( metabgi.bgiFont != metabgi.defFont )
         _graphfreemem( metabgi.bgiFont, 0 );
      
      // set to internal font in case of errors
      metabgi.bgiFont = metabgi.defFont;


      // check for user supplied font
 	   if( font < maxStdFonts )
         strcpy( buf, fontFile[font] );
      else {

         // compute user font index
         i = font - maxStdFonts;

         // is user font index empty ?
         if( metabgi.userfnt[i][0] == '\0' ) {
            font = 0;
            strcpy( buf, fontFile[font] );
            metabgi.bgiErrCode = grInvalidFontNum;
         }

         // is it registered ?
         if( metabgi.userfnt[i][0] == 1 ) {
            registered = 1;
            // get pointer from next far * of buffer
            p = ( void **) &metabgi.userfnt[i][1];
            SetFont( ( fontRcd * ) *p  );
            return;  // done!
         }

         // must be installed
         strcpy( buf, &(metabgi.userfnt[i][0]) );

      } 

      // if bitmapped font, build name from size and direction
      if( font == 0 ) {
         buf[6] = (charsize / 10) + 0x30;
         buf[7] = (charsize % 10) + 0x30;
         if( direction )
            buf[3] = 'R';
      }      

      // we now have a font file name in buf
      
      // see if we can locate the font file
      i = ResrcQuery ( buf, &fileInfo, 1 );

      // did we locate a file?
      if( i == 1 )  {

         // allocate space for the new font
         newfnt = (fontRcd *) _graphgetmem( fileInfo.fileSize );

         // did we get memory ?
         if( newfnt ) {
            i = ResrcLoad ( buf, newfnt, fileInfo.fileSize );
            // did we get a file ?
            if( i >= 0 )
               metabgi.bgiFont = newfnt;   
         }
         else // can't get memory
            metabgi.bgiErrCode = grNoFontMem;
         
      }
      else  // can't find file
         metabgi.bgiErrCode = grFontNotFound;

      SetFont ( metabgi.bgiFont ); 				

   }  // end of should we load a new font file


   // process the "direction" setting
   if ( metabgi.textsettings.direction != direction ) {   // new direction?

      if ( direction == VERT_DIR ) {
         TextPath ( 900 );  // VERT_DIR 
         // BGI alignment is not relative to path
         TextAlign ( metabgi.textsettings.vert, metabgi.textsettings.horiz + 1 );
      }
      else {
         TextPath (   0 );  // HORZ_DIR
         // BGI alignment is not relative to path
         TextAlign ( metabgi.textsettings.horiz, metabgi.textsettings.vert + 1 );
      }

   }

   // process the "charsize" setting
   if ( metabgi.textsettings.charsize != charsize )  {    // new charsize?

      // user size ?
      if ( charsize == 0 ) {
         // pickup the fonts 'size'
         i = metabgi.bgiFont->fontPtSize;
         TextSize ( i * metabgi.bgiMultX / metabgi.bgiDivX,
                    i * metabgi.bgiMultY / metabgi.bgiDivY );
      }

      // hardwired size ?
      if ( charsize > 0  && charsize < 11 ) {
         // pickup the fonts 'size'
         i = metabgi.bgiFont->fontPtSize;
         // scale as per table
         i *= strksz[charsize].X;
         i /= strksz[charsize].Y;
         TextSize ( i, i );
      }
   }

   // always proportional
   TextFace ( cProportional );

   // save the new text settings
   metabgi.textsettings.direction = direction;
   metabgi.textsettings.charsize  = charsize;

}


/* End of File - MBSETTXS.C */

